from deepface import DeepFace
import pandas as pd
import numpy as np
import cv2 as cv
import os

models = [
    "VGG-Face",
    "Facenet",
    "Facenet512",
    "OpenFace",
    # "DeepFace",
    "DeepID",
    "ArcFace",
    # "Dlib",
    "SFace",
    "GhostFaceNet"
]

results = {
    "model": models,
    "original": [],
    "rotated_45": [],
    "rotated_90": [],
    "blurred": [],
    "dark": [],
    "bright": [],
    "noisy": []
}


def augment_dataset():
    dataset_folder = "./task3_2_input/dataset"
    original_folder = os.path.join(dataset_folder, "original")
    for filename in os.listdir(original_folder):
        image_path = os.path.join(original_folder, filename)
        image = cv.imread(image_path)
        if image is not None:
            print(f"Processing {filename}")

            # Rotate 45 degrees
            (h, w) = image.shape[:2]
            center = (w // 2, h // 2)
            M = cv.getRotationMatrix2D(center, 45, 1.0)
            rotated_45 = cv.warpAffine(image, M, (w, h))

            os.makedirs(os.path.join(dataset_folder, "rotated_45"), exist_ok=True)
            rotated_45_path = os.path.join(
                dataset_folder,
                "rotated_45",
                f"{os.path.splitext(filename)[0]}_rotated_45{os.path.splitext(filename)[1]}"
            )
            cv.imwrite(rotated_45_path, rotated_45)

            # Rotate 90 degrees
            rotated_90 = cv.rotate(image, cv.ROTATE_90_CLOCKWISE)
            os.makedirs(os.path.join(dataset_folder, "rotated_90"), exist_ok=True)
            rotated_90_path = os.path.join(
                dataset_folder,
                "rotated_90",
                f"{os.path.splitext(filename)[0]}_rotated_90{os.path.splitext(filename)[1]}"
            )
            cv.imwrite(rotated_90_path, rotated_90)

            # Blur
            blurred = cv.GaussianBlur(image, (15, 15), 0)
            os.makedirs(os.path.join(dataset_folder, "blurred"), exist_ok=True)
            blurred_path = os.path.join(
                dataset_folder,
                "blurred",
                f"{os.path.splitext(filename)[0]}_blurred{os.path.splitext(filename)[1]}"
            )
            cv.imwrite(blurred_path, blurred)

            # Brightness
            # Decrease brightness by 50%
            dark = cv.convertScaleAbs(image, alpha=0.5, beta=0)
            os.makedirs(os.path.join(dataset_folder, "dark"), exist_ok=True)
            dark_path = os.path.join(
                dataset_folder,
                "dark",
                f"{os.path.splitext(filename)[0]}_dark{os.path.splitext(filename)[1]}"
            )
            cv.imwrite(dark_path, dark)

            # Increase brightness by 50%
            bright = cv.convertScaleAbs(image, alpha=1.5, beta=0)
            os.makedirs(os.path.join(dataset_folder, "bright"), exist_ok=True)
            bright_path = os.path.join(
                dataset_folder,
                "bright",
                f"{os.path.splitext(filename)[0]}_bright{os.path.splitext(filename)[1]}"
            )
            cv.imwrite(bright_path, bright)

            # Add noise
            noise = image.copy()
            mean = 0
            sigma = 25
            gauss = np.random.normal(mean, sigma, noise.shape).astype("uint8")
            noisy_image = cv.add(noise, gauss)
            os.makedirs(os.path.join(dataset_folder, "noisy"), exist_ok=True)
            noisy_path = os.path.join(
                dataset_folder,
                "noisy",
                f"{os.path.splitext(filename)[0]}_noisy{os.path.splitext(filename)[1]}"
            )
            cv.imwrite(noisy_path, noisy_image)
        else:
            print(f"Failed to load {filename}")


def main():
    # augment_dataset()
    
    for model in models:
        print("==================================")
        print(f"Processing model {model}")

        for dataset in list(results.keys())[1:]:
            detection = DeepFace.find(
                img_path="task3_2_input/egor.jpg",
                db_path=f"task3_2_input/dataset/{dataset}",
                model_name=model,
                enforce_detection=False
            )
            recall = float(len(detection[0])) / 20.0
            results[dataset].append(recall)

            print(len(detection[0]))

        print()

    table = pd.DataFrame(results)
    print(table)
    table.to_csv("task3_2_output/results.txt", index=False)


if __name__ == "__main__":
    main()
